﻿Type.registerNamespace("Jeffz.UI");

Jeffz.UI.AutoCompleteBehavior = function()
{
	Jeffz.UI.AutoCompleteBehavior.initializeBase(this);
	
	var _appURL;
	var _serviceURL;
	var _serviceMethod;
	var _minimumPrefixLength = 3;
	var _completionSetCount = 10;
	var _completionInterval = 1000;
	var _completionListElement;
	var _popupBehavior;
	var _itemTemplate;
	var _completionListCssClass;
	var _itemCssClass;
	var _highlightedItemCssClass;
	var _textProperty;
	
	var _timer;
	var _cache;
	var _currentPrefix;
	var _selectIndex;
	
	var _focusHandler;
	var _blurHandler;
	var _keyDownHandler;
	var _mouseDownHandler;
	var _mouseUpHandler;
	var _mouseOverHandler;
	var _tickHandler;
	
	var _additionalOffset = (Sys.Runtime.get_hostType() == Sys.HostType.InternetExplorer) ? 0 : 2;
	
	this.get_appURL = function()
	{
		return _appURL;
	}
	this.set_appURL = function(value)
	{
		_appURL = value;
	}

	this.get_completionInterval = function()
	{
		return _completionInterval;
	}
	this.set_completionInterval = function(value)
	{
		_completionInterval = value;
	}
	
	this.get_completionList = function()
	{
		return _completionListElement;
	}
	this.set_completionList = function(value)
	{
		_completionListElement = value;
	}
	
	this.get_completionSetCount = function()
	{
		return _completionSetCount;
	}
	this.set_completionSetCount = function(value)
	{
		_completionSetCount = value;
	}
	
	this.get_minimumPrefixLength = function()
	{
		return _minimumPrefixLength;
	}
	this.set_minimumPrefixLength = function(value)
	{
		_minimumPrefixLength = value;
	}
	
	this.get_serviceMethod = function()
	{
		return _serviceMethod;
	}
	this.set_serviceMethod = function(value)
	{
		_serviceMethod = value;
	}
	
	this.get_serviceURL = function()
	{
		return _serviceURL;
	}
	this.set_serviceURL = function(value)
	{
		_serviceURL = value;
	}
	
	this.get_itemTemplate = function()
	{
		return _itemTemplate;
	}
	this.set_itemTemplate = function(value)
	{
		if (_itemTemplate)
		{
			_itemTemplate.dispose();
		}
		_itemTemplate = value;
		
		this.raisePropertyChanged('itemTemplate');
	}
	
	this.get_completionListCssClass = function()
	{
		return _completionListCssClass;
	}
	this.set_completionListCssClass = function(value)
	{
		if (_completionListCssClass != value)
		{
			_completionListCssClass = value;
			this.raisePropertyChanged('completionListCssClass');
		}
	}
	
	this.get_highlightedItemCssClass = function()
	{
		return _highlightedItemCssClass;
	}
	this.set_highlightedItemCssClass = function(value)
	{
		if (_highlightedItemCssClass != value)
		{
			_highlightedItemCssClass = value;
			this.raisePropertyChanged('highlightedItemCssClass');
		}
	}
	
	this.get_itemCssClass = function()
	{
		return _itemCssClass;
	}
	this.set_itemCssClass = function(value)
	{
		if (_itemCssClass != value)
		{
			_itemCssClass = value;
			this.raisePropertyChanged('itemCssClass');
		}
	}
	
	this.get_textProperty = function()
	{
		return _textProperty;
	}
	this.set_textProperty = function(value)
	{
		if (_textProperty != value)
		{
			_textProperty = value;
			this.raisePropertyChanged('textProperty');
		}
	}

	this.dispose = function()
	{
		if (_timer) {
			_timer.tick.remove(_tickHandler);
			_timer.dispose();
		}
		
		var element = this.control.element;
		element.detachEvent('onfocus', _focusHandler);
		element.detachEvent('onblur', _blurHandler);
		element.detachEvent('onkeydown', _keyDownHandler);
		
		if (_itemTemplate) {
			_itemTemplate.dispose();
		}
		
		_tickHandler = null;
		_focusHandler = null;
		_blurHandler = null;
		_keyDownHandler = null;
		_mouseDownHandler = null;
		_mouseUpHandler = null;
		_mouseOverHandler = null;

		Jeffz.UI.AutoCompleteBehavior.callBaseMethod(this, 'dispose');
	}

	this.getDescriptor = function()
	{
		var td = Jeffz.UI.AutoCompleteBehavior.callBaseMethod(this, 'getDescriptor');
		td.addProperty('completionInterval', Number);
		td.addProperty('completionList', Object, false, Sys.Attributes.Element, true);
		td.addProperty('completionSetCount', Number);
		td.addProperty('minimumPrefixLength', Number);
		td.addProperty('serviceMethod', String);
		td.addProperty('serviceURL', String);
		td.addProperty('appURL', String);
		td.addProperty('itemTemplate', Sys.UI.ITemplate);
		td.addProperty('completionListCssClass', String);
		td.addProperty('itemCssClass', String);
		td.addProperty('highlightedItemCssClass', String);
		td.addProperty('textProperty', String);
		return td;
	}
	
	this.initialize = function()
	{
		Jeffz.UI.AutoCompleteBehavior.callBaseMethod(this, 'initialize');

		_tickHandler = Function.createDelegate(this, this._onTimerTick);
		_focusHandler = Function.createDelegate(this, this._onGotFocus);
		_blurHandler = Function.createDelegate(this, this._onLostFocus);
		_keyDownHandler = Function.createDelegate(this, this._onKeyDown);
		_mouseDownHandler = Function.createDelegate(this, this._onListMouseDown);
		_mouseUpHandler = Function.createDelegate(this, this._onListMouseUp);
		_mouseOverHandler = Function.createDelegate(this, this._onListMouseOver);
		
		_timer = new Sys.Timer();
		_timer.set_interval(_completionInterval);
		_timer.tick.add(_tickHandler);
		
		if (_itemTemplate) {
			_itemTemplate.initialize();
		}
		
		var element = this.control.element;
		element.autocomplete = "off";
		element.attachEvent('onfocus', _focusHandler);
		element.attachEvent('onblur', _blurHandler);
		element.attachEvent('onkeydown', _keyDownHandler);
		
		if (!_completionListElement) {
			_completionListElement = document.createElement('DIV');
			document.body.appendChild(_completionListElement);
		}
		
		document.body.appendChild(_completionListElement);
		var popupControl = new Sys.UI.Control(_completionListElement);
		_popupBehavior = new Sys.UI.PopupBehavior();
		_popupBehavior.set_parentElement(element);
		_popupBehavior.set_positioningMode(Sys.UI.PositioningMode.BottomLeft);
		popupControl.get_behaviors().add(_popupBehavior);
		_popupBehavior.initialize();
		popupControl.initialize();
	}
	
	this._hideCompletionList = function()
	{
		_popupBehavior.hide();
		_completionListElement.innerHTML = '';
		_selectIndex = -1;
	}
	
	this._highlightItem = function(item, oldIndex)
	{
		if (_itemCssClass && oldIndex >= 0)
		{
			_completionListElement.childNodes[oldIndex].className = _itemCssClass;
		}
		
		if (_highlightedItemCssClass)
		{
			item.className = _highlightedItemCssClass;
		}
		
		if (_completionListCssClass)
		{
			_completionListElement.className = _completionListCssClass;
		}
	}
	
	this._onListMouseDown = function()
	{
		this._setText(findItemElement(window.event.srcElement)._completedText);
	}
	
	this._onListMouseUp = function()
	{
		this.control.focus();
	}
	
	this._onListMouseOver = function()
	{
		var item = findItemElement(window.event.srcElement);
		this._highlightItem(item, _selectIndex);
		_selectIndex = item.selectIndex;
	}

	this._onGotFocus = function()
	{
		_timer.set_enabled(true);
	}
	
	this._onKeyDown = function()
	{
		var e = window.event;
		if (e.keyCode == 27)
		{
			this._hideCompletionList();
			e.returnValue = false;
		}
		else if (e.keyCode == Sys.UI.Key.Up)
		{
			if (_selectIndex > 0)
			{
				this._highlightItem(_completionListElement.childNodes[_selectIndex - 1], _selectIndex);
				_selectIndex--;
				e.returnValue = false;
			}
		}
		else if (e.keyCode == Sys.UI.Key.Down)
		{
			if (_selectIndex < (_completionListElement.childNodes.length - 1))
			{
				this._highlightItem(_completionListElement.childNodes[_selectIndex + 1], _selectIndex);
				_selectIndex++;
				e.returnValue = false;
			}
		}
		else if (e.keyCode == Sys.UI.Key.Return)
		{
			if (_selectIndex != -1) {
				this._setText(_completionListElement.childNodes[_selectIndex]._completedText);
				e.returnValue = false;
			}
		}
		
		if (e.keyCode != Sys.UI.Key.Tab)
		{
			_timer.set_enabled(true);
		}
	}
	
	this._onLostFocus = function()
	{
		_timer.set_enabled(false);
		this._hideCompletionList();
	}
	
	function findItemElement(ele)
	{
		while (ele.parentNode != _completionListElement)
		{
			ele = ele.parentNode;
		}
		
		return ele;
	}
	
	function _onMethodComplete(result, response, context)
	{
		var acBehavior = context[0];
		var prefixText = context[1];
		acBehavior._update(prefixText, result,  true);
	}
	
	this._onTimerTick = function(sender, eventArgs)
	{
		if (_serviceURL && _serviceMethod) {
			var text = this.control.element.value;
			
			if (text.trim().length < _minimumPrefixLength)
			{
				this._update('', null,  false);
				return;
			}
			
			if (_currentPrefix != text) {
				_currentPrefix = text;
				if (_cache && _cache[text]) {
					this._update(text, _cache[text],  false);
					return;
				}
				
				Sys.Net.ServiceMethod.invoke(
					_serviceURL,
					_serviceMethod,
					_appURL,
					{ prefixText : _currentPrefix, count: _completionSetCount },
					_onMethodComplete, null, null, null,
					[ this, text ]);
			}
		}
	}
	
	this._setText = function(text)
	{
		_timer.set_enabled(false);
		_currentPrefix = text;
		if (Sys.UI.TextBox.isInstanceOfType(this.control)) {
			this.control.set_text(text);
		}
		else {
			this.control.element.value = text;
		}
		this._hideCompletionList();
	}
	
	this._update = function(prefixText, completionItems, cacheResults)
	{
		if (cacheResults) {
			if (!_cache) {
				_cache = { };
			}
			_cache[prefixText] = completionItems;
		}

		_completionListElement.innerHTML = '';
		if (_completionListCssClass)
		{
			_completionListElement.className = _completionListCssClass;
		}
		
		_selectIndex = -1;
		if (completionItems && completionItems.length)
		{
			for (var i = 0; i < completionItems.length; i++)
			{
				var newElement = _itemTemplate.createInstance(_completionListElement, completionItems[i]).instanceElement;
				Sys.UI.ITemplate.disposeInstance(newElement);
				
				newElement.attachEvent('onmousedown', _mouseDownHandler);
				newElement.attachEvent('onmouseup', _mouseUpHandler);
				newElement.attachEvent('onmouseover', _mouseOverHandler);
				newElement.selectIndex = i;
				newElement._completedText = completionItems[i][_textProperty];
				
				if (_itemCssClass)
				{
					newElement.className = _itemCssClass;
				}
			}
			
			_completionListElement.style.width = 
				Sys.UI.Control.getBounds(this.control.element).width - 2 - _additionalOffset + "px";
			_completionListElement.style.overflow = "hidden";
			_completionListElement.style.borderWidth = "1px";
			_popupBehavior.show();
		}
		else
		{
			_popupBehavior.hide();
		}
	}
}
Jeffz.UI.AutoCompleteBehavior.registerSealedClass('Jeffz.UI.AutoCompleteBehavior', Sys.UI.Behavior);
Sys.TypeDescriptor.addType('jeffz', 'autoComplete', Jeffz.UI.AutoCompleteBehavior);